package com.qmth.teachcloud.report.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.report.business.bean.result.*;
import com.qmth.teachcloud.report.business.entity.*;
import com.qmth.teachcloud.report.business.enums.PublishStatusEnum;
import com.qmth.teachcloud.report.business.enums.SemesterEnum;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseMapper;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseRecordMapper;
import com.qmth.teachcloud.report.business.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 * 考试课程维度分析表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-06-01
 */
@Service
public class TAExamCourseServiceImpl extends ServiceImpl<TAExamCourseMapper, TAExamCourse> implements TAExamCourseService {

    @Resource
    TAExamCourseMapper taExamCourseMapper;

    @Resource
    TAExamCourseRecordMapper taExamCourseRecordMapper;

    @Resource
    TBPaperService tbPaperService;

    @Resource
    TAExamCourseCollegeInspectService taExamCourseCollegeInspectService;

    @Resource
    TBDimensionService tbDimensionService;

    @Resource
    TBCommonLevelConfigService tbCommonLevelConfigService;

    @Resource
    TBPaperStructService tbPaperStructService;

    @Resource
    TBAnswerService tbAnswerService;

    @Resource
    TAExamCourseCollegeTeacherService taExamCourseCollegeTeacherService;

    /**
     * 开课课程考试总览列表接口
     *
     * @param iPage
     * @param examId
     * @param semester
     * @param schoolId
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyTeacherList(IPage<Map> iPage, Long examId, SemesterEnum semester, Long schoolId) {
        return taExamCourseMapper.surveyTeacherList(iPage, examId, semester.name(), schoolId);
    }


    @Override
    public TAExamCourse getOverview(String semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return taExamCourseMapper.getOverview(schoolId, semester, examId, courseCode);
    }

    /**
     * 赋分管理列表接口
     *
     * @param iPage
     * @param schoolId
     * @param examId
     * @param courseCode
     * @param publishStatus
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyAspointsList(IPage<Map> iPage, Long schoolId, Long examId, String courseCode, PublishStatusEnum publishStatus) {
        return taExamCourseMapper.surveyAspointsList(iPage, schoolId, examId, courseCode, publishStatus.name());
    }

    @Override
    public Map<String, Object> getGradeDistribute(String semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Map<String, Object> map = new HashMap<>();
        // 不及格率
        Map<String, String> scoreAndRate = taExamCourseMapper.getScoreAndRate(schoolId, semester, examId, courseCode);
        map.put("scoreAndRate", scoreAndRate);
        // 成绩正态分布
        String[] strings = {"90~100", "80~89", "70~79", "60~69", "55~59", "1~54"};
        List<Map<String, Object>> mapList = new ArrayList<>();
        for (String string : strings) {
            String[] str = string.split("~");
            double startScore = Double.parseDouble(str[0]);
            double endScore = Double.parseDouble(str[1]);
            List<TAExamCourseRecord> taExamCourseRecords = taExamCourseRecordMapper.listExamCourseRecord(schoolId, semester, examId, courseCode);
            // 全部有效人数
            int totalRealityCount = taExamCourseRecords.size();
            // 本分数段人数
            long totalScoreCount = taExamCourseRecords.stream().filter(m -> m.getAssignedScore().doubleValue() >= startScore && m.getAssignedScore().doubleValue() <= endScore).count();
            // 应届有效人数
            long totalCurrentCount = taExamCourseRecords.stream().filter(TAExamCourseRecord::getStudentCurrent).count();
            // 本分数段应届有效人数
            long totalScoreCurrentCount = taExamCourseRecords.stream().filter(m -> m.getStudentCurrent() && m.getAssignedScore().doubleValue() >= startScore && m.getAssignedScore().doubleValue() <= endScore).count();
            Map<String, Object> objectMap = new HashMap<>();
            objectMap.put("scores", String.join(",", str));
            objectMap.put("totalRealityCount", totalRealityCount);
            objectMap.put("totalRate", totalRealityCount == 0 ? 0 : new BigDecimal(totalScoreCount).divide(new BigDecimal(totalRealityCount), 2, BigDecimal.ROUND_HALF_UP));
            objectMap.put("currentRealityCount", totalCurrentCount);
            objectMap.put("totalRate", totalCurrentCount == 0 ? 0 : new BigDecimal(totalScoreCurrentCount).divide(new BigDecimal(totalCurrentCount), 2, BigDecimal.ROUND_HALF_UP));
            mapList.add(objectMap);
        }
        map.put("grades", mapList);
        return map;
    }

    /**
     * 考查课程考试分析接口
     *
     * @param examId
     * @param semester
     * @param courseCode
     * @param schoolId
     * @return
     */
    @Override
    public List<Object> surveyInspectAnalyseView(Long examId, SemesterEnum semester, String courseCode, Long schoolId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        //查找试卷
        QueryWrapper<TBPaper> tbPaperQueryWrapper = new QueryWrapper<>();
        tbPaperQueryWrapper.lambda().eq(TBPaper::getExamId, examId)
                .eq(TBPaper::getCourseCode, courseCode);
        List<TBPaper> tbPaperList = tbPaperService.list(tbPaperQueryWrapper);

        //查找科目学院维度
        List<TAExamCourseCollegeInspectResult> taExamCourseCollegeInspectResultList = taExamCourseCollegeInspectService.findCourseDescriptiveStatisticsForCollege(examId, courseCode);

        //查找维度
        List<CourseDimensionResult> courseDimensionResultList = tbDimensionService.findDimensionInfo(examId, courseCode, sysUser.getOrgId());

        //查找难易度
        QueryWrapper<TBCommonLevelConfig> tbCommonLevelConfigQueryWrapper = new QueryWrapper<>();
        tbCommonLevelConfigQueryWrapper.lambda().eq(TBCommonLevelConfig::getExamId, examId)
                .eq(TBCommonLevelConfig::getCourseCode, courseCode)
                .eq(TBCommonLevelConfig::getLevelType, "难度等级");
        List<TBCommonLevelConfig> tbCommonLevelConfigList = tbCommonLevelConfigService.list(tbCommonLevelConfigQueryWrapper);

        //查找试卷结构
        List<TBPaperStructResult> tbPaperStructList = tbPaperStructService.findQuestionInfo(examId, courseCode);

        //查找答题记录
        List<TBAnswerResult> tbAnswerResultList = tbAnswerService.findValidAnswerDetail(examId, courseCode);

        //查找老师得分
        List<TAExamCourseTeacherResult> taExamCourseTeacherResultList = taExamCourseCollegeTeacherService.findTeacherInfo(examId, courseCode, sysUser.getOrgId());

        //查找维度
        QueryWrapper<TBDimension> tbDimensionQueryWrapper = new QueryWrapper<>();
        tbDimensionQueryWrapper.lambda().eq(TBDimension::getExamId, examId)
                .eq(TBDimension::getCourseCode, courseCode);
        List<TBDimension> tbDimensionList = tbDimensionService.list(tbDimensionQueryWrapper);
        return null;
    }
}
